/*******************************************************
**
** Implemention of QyurSqlTreeView
**
** Copyright (C) 2009-2013 Yuri Yurachkovsky.
**
** QyurSqlTreeView is free software; you can redistribute
** it and/or modify it under the terms of the GNU
** Library General Public License as published by the
** Free Software Foundation; either version 2, or (at
** your option) any later version.
**
********************************************************/
#include "qyursqltreeview.h"
#include <QtCore>
#include <QtGui>
#include <QtSql>

namespace {
/**************************************************************************************************
** Important: source model is always sorted asc by parid column. Then real user sort is perfomes 
** in context of parid sort. class QSqlTableModelEx take care about it;
** We expect also that QSqlTableModel have QSqlTableModel::OnManualSubmit editStrategy, else can not
** work normally.
***************************************************************************************************/
const int DROP_MULTIORDER=-2;
class QSqlTableModelEx: public QSqlTableModel {
	Q_OBJECT
public:
	QSqlTableModelEx(QObject* parent=0, QSqlDatabase db = QSqlDatabase());
	void setSort(int column=-1, Qt::SortOrder order=Qt::AscendingOrder);
protected:
	struct SortingStruct {
		SortingStruct(int field_=0, Qt::SortOrder sortingOrder_=Qt::AscendingOrder) {
			field=field_;
			sortingOrder=sortingOrder_;
		}
		int field;
		Qt::SortOrder sortingOrder;
	};
	QString orderByClause() const;
	QVector<SortingStruct> sortingFields;
};

QSqlTableModelEx::QSqlTableModelEx(QObject* parent, QSqlDatabase db):QSqlTableModel(parent, db) {
	setEditStrategy(QSqlTableModel::OnManualSubmit);
}

QString QSqlTableModelEx::orderByClause() const {
	QString s;
	if (sortingFields.isEmpty())
		return s;
	s.append("ORDER BY ");
	foreach(SortingStruct sortingField,sortingFields)
		s.append(record().fieldName(sortingField.field)).append(sortingField.sortingOrder==Qt::AscendingOrder?" ASC":" DESC").append(",");
	s.chop(1);
	return s;
}

void QSqlTableModelEx::setSort(int column, Qt::SortOrder order) {
	if (!record().field(column).isValid())
		return;
	if (order==DROP_MULTIORDER) {
		sortingFields.clear();
		sortingFields.append(SortingStruct(column,Qt::AscendingOrder));
		emit headerDataChanged(Qt::Horizontal,0,columnCount()-1);//update icons
	}
	else {
		for(int i=0; i<sortingFields.size();i++)
			if (sortingFields[i].field==column) {
				sortingFields[i].sortingOrder= (Qt::SortOrder)!sortingFields[i].sortingOrder;//order;
				return;
			}
		sortingFields.append(SortingStruct(column,order));
	}
}

struct UserData {
	UserData(int id, int parid):id(id),parid(parid) {
	}
	~UserData() {
	}
	int id,parid;
};
}//namespace

class QyurSqlTreeModelPrivate {
	Q_DECLARE_PUBLIC(QyurSqlTreeModel)
	QyurSqlTreeModel* q_ptr;
	mutable QMap<int,int> id2row,parid2row,id2ParentRow,id2rowCount;
	mutable QMap<QPair<int,int>,int> row2id;
	mutable QMap<int,UserData*> id2UserData;
	mutable QMap<QModelIndex,QModelIndex> proxy2source;
	mutable QMap<int,int> proxyColumn2Original;
	
	QMap<QObject*,QSet<int> > expandedSet;
	QMap<QObject*,QSet<int> > selectedSet;
	QMap<QObject*,int> current;
public:
	QSqlTableModelEx sourceModel;
	int indexOfId, indexOfParid, rootParentId;
	int originalColumnByProxy(int proxyColumn) const {
		return proxyColumn2Original.value(proxyColumn,proxyColumn);
	}
	int getRowById(int) const;
	int getRowByParid(int) const;
	UserData* getUserDataById(int) const;
	QyurSqlTreeModelPrivate() {

	}
	~QyurSqlTreeModelPrivate() {
		clear();
	}
	void clear() {
		qDeleteAll(id2UserData);
		id2row.clear();
		parid2row.clear();
		id2ParentRow.clear();
		id2rowCount.clear();
		row2id.clear();
		id2UserData.clear();
		proxy2source.clear();
	}
}; 

UserData* QyurSqlTreeModelPrivate::getUserDataById(int id) const {
	if (!id2UserData[id]) {
		int parid= sourceModel.record(getRowById(id)).field(indexOfParid).value().toInt();
		id2UserData[id]= new UserData(id,parid);
	}
	return id2UserData[id];
}

int QyurSqlTreeModelPrivate::getRowById(int id) const {
	if (!id2row.contains(id) || sourceModel.index(id2row[id],indexOfId).data().toInt()!=id) {
		for (int i=0; i<sourceModel.rowCount(); i++)
			if (sourceModel.record(i).field(indexOfId).value().toInt()==id) {
				id2row[id]= i;
				break;
			}
	} 
	return id2row.value(id,-1);
}

int QyurSqlTreeModelPrivate::getRowByParid(int parid) const {
	if (!parid2row.contains(parid) || sourceModel.index(parid2row[parid],indexOfParid).data().toInt()!=parid || (parid2row[parid]&&sourceModel.index(parid2row[parid]-1,indexOfParid).data().toInt()==parid) ) {
		for (int i=0; i<sourceModel.rowCount(); i++)
			if (sourceModel.record(i).field(indexOfParid).value().toInt()==parid) {
				parid2row[parid]= i;
				break;
			}
	}
	return parid2row.value(parid,-1);
}

QyurSqlTreeModel::QyurSqlTreeModel(const QString& tableName, const QStringList& captions, const QStringList& fieldNames, const QString& idField, const QString& parentIdField, /*const QString& decoratedField, */int rootParentId, QObject* parent):QAbstractProxyModel(parent),d_ptr(new QyurSqlTreeModelPrivate) {
	Q_D(QyurSqlTreeModel);
	d->q_ptr= this;
	d->rootParentId= rootParentId;
	setSourceModel(&d->sourceModel);
	d->sourceModel.setTable(tableName);
	d->indexOfId= d->sourceModel.record().indexOf(idField);
	Q_ASSERT(d->indexOfId>=0);
	d->indexOfParid= d->sourceModel.record().indexOf(parentIdField);
	Q_ASSERT(d->indexOfParid>=0);
	for (int i=0; i<qMin(fieldNames.size(),captions.size()); i++)
		d->sourceModel.setHeaderData(d->sourceModel.record().indexOf(fieldNames[i]),Qt::Horizontal,captions[i]);
	for (int i=0; i<fieldNames.count();i++) {
		d->proxyColumn2Original[i]= d->sourceModel.record().indexOf(fieldNames[i]);
	}
	d->sourceModel.setSort(d->indexOfParid ,Qt::AscendingOrder);
	d->sourceModel.select();
}

QModelIndex QyurSqlTreeModel::addChild(const QVariantList& values, const QModelIndex& parent, QAbstractItemView* treeView) {
	Q_D(QyurSqlTreeModel);
	QSqlTableModel forInsertModel;
	connect(&forInsertModel, SIGNAL(beforeInsert(QSqlRecord&)),getSourceModel(), SIGNAL(beforeInsert(QSqlRecord&)));
	forInsertModel.setTable(getSourceModel()->tableName());
	forInsertModel.setEditStrategy(QSqlTableModel::OnManualSubmit);
	forInsertModel.select();
	QSqlRecord record= forInsertModel.record();
	for (int i=0; i<values.size(); i++)
		record.setValue(i, values[i]);
	record.setValue(getSourceParentIdColumn(),getIdByIndex(parent));
	forInsertModel.insertRecord(-1,record);
	forInsertModel.submitAll();
	int id=  forInsertModel.record(forInsertModel.rowCount()-1).field(getSourceIdColumn()).value().toInt();
	if (id>0) {
		d->current[treeView?treeView->selectionModel():0]= id;
		refresh();
		return getIndexById(id);
	}
	return QModelIndex();
}

QModelIndexList QyurSqlTreeModel::expandedList(QObject* o) const {
	Q_D(const QyurSqlTreeModel);
	QModelIndexList miList;
	foreach(int id, d->expandedSet[o]) {
		miList<<getIndexById(id);
	}
	return miList;
}

QModelIndexList QyurSqlTreeModel::selectionList(QObject* o) const {
	Q_D(const QyurSqlTreeModel);
	QModelIndexList miList;
	foreach(int id, d->selectedSet[o]) {
		miList<<getIndexById(id);
	}
	return miList;
}

QModelIndex QyurSqlTreeModel::current(QObject* o) const {
	Q_D(const QyurSqlTreeModel);
	return getIndexById(d->current[o]);
}

void QyurSqlTreeModel::addExpanded(const QModelIndex& index) {
	Q_D(QyurSqlTreeModel);
	d->expandedSet[sender()]+= getIdByIndex(index);
}

void QyurSqlTreeModel::addCollapsed(const QModelIndex& index) {
	Q_D(QyurSqlTreeModel);
	d->expandedSet[sender()]-= getIdByIndex(index);
}

void QyurSqlTreeModel::onSelectionChanged(const QItemSelection& selected, const QItemSelection& deselected) {
	Q_UNUSED(deselected);
	Q_D(QyurSqlTreeModel);
	d->selectedSet.clear();
	QModelIndexList miList= selected.indexes();
	foreach(QModelIndex index, miList)
		d->selectedSet[sender()]+= getIdByIndex(index);
}

void QyurSqlTreeModel::onCurrentChanged(const QModelIndex& current) {
	Q_D(QyurSqlTreeModel);
	d->current[sender()]= getIdByIndex(current);
}

int QyurSqlTreeModel::originalColumnByProxy(int proxyColumn) const {
	Q_D(const QyurSqlTreeModel);
	return d->originalColumnByProxy(proxyColumn);
}

int QyurSqlTreeModel::proxyColumnByOriginal(int originalColumn) const {
	Q_D(const QyurSqlTreeModel);
	foreach(int proxy,d->proxyColumn2Original) {
		if (d->proxyColumn2Original[proxy]==originalColumn)
			return proxy;
	}
	return originalColumn;
}

int QyurSqlTreeModel::proxyColumnByOriginal(const QString& field) const {
	Q_D(const QyurSqlTreeModel);
	return proxyColumnByOriginal(d->sourceModel.fieldIndex(field));
}

QVariant QyurSqlTreeModel::headerData(int section, Qt::Orientation orientation, int role) const {
	#if QT_VERSION >= 0x040400
	return QAbstractProxyModel::headerData(section,orientation,role);
	#else
	Q_D(const QyurSqlTreeModel);
	return QAbstractProxyModel::headerData(d->originalColumnByProxy(section),orientation,role);
	#endif
}
/*
void QyurSqlTreeModel::setSourceModel(QAbstractItemModel *sourceModel) {
	Q_D(QyurSqlTreeModel);
	disconnect(getSourceModel(), SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(sourceDataChanged(QModelIndex,QModelIndex)));
	disconnect(getSourceModel(), SIGNAL(headerDataChanged(Qt::Orientation,int,int)), this, SLOT(sourceHeaderDataChanged(Qt::Orientation,int,int)));
	disconnect(getSourceModel(), SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)), this, SLOT(sourceRowsAboutToBeInserted(QModelIndex,int,int)));
	disconnect(getSourceModel(), SIGNAL(rowsInserted(QModelIndex,int,int)), this, SLOT(sourceRowsInserted(QModelIndex,int,int)));
	disconnect(getSourceModel(), SIGNAL(columnsAboutToBeInserted(QModelIndex,int,int)), this, SLOT(sourceColumnsAboutToBeInserted(QModelIndex,int,int)));
	disconnect(getSourceModel(), SIGNAL(columnsInserted(QModelIndex,int,int)), this, SLOT(sourceColumnsInserted(QModelIndex,int,int)));
	disconnect(getSourceModel(), SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),this, SLOT(sourceRowsAboutToBeRemoved(QModelIndex,int,int)));
	disconnect(getSourceModel(), SIGNAL(rowsRemoved(QModelIndex,int,int)), this, SLOT(sourceRowsRemoved(QModelIndex,int,int)));
	disconnect(getSourceModel(), SIGNAL(columnsAboutToBeRemoved(QModelIndex,int,int)), this, SLOT(sourceColumnsAboutToBeRemoved(QModelIndex,int,int)));
	disconnect(getSourceModel(), SIGNAL(columnsRemoved(QModelIndex,int,int)), this, SLOT(sourceColumnsRemoved(QModelIndex,int,int)));
	disconnect(getSourceModel(), SIGNAL(layoutAboutToBeChanged()), this, SLOT(sourceLayoutAboutToBeChanged()));
	disconnect(getSourceModel(), SIGNAL(layoutChanged()), this, SLOT(sourceLayoutChanged()));
	disconnect(getSourceModel(), SIGNAL(modelReset()), this, SLOT(sourceReset()));
	QAbstractProxyModel::setSourceModel(sourceModel);
	connect(getSourceModel(), SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(sourceDataChanged(QModelIndex,QModelIndex)));
	connect(getSourceModel(), SIGNAL(headerDataChanged(Qt::Orientation,int,int)), this, SLOT(sourceHeaderDataChanged(Qt::Orientation,int,int)));
	connect(getSourceModel(), SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)), this, SLOT(sourceRowsAboutToBeInserted(QModelIndex,int,int)));
	connect(getSourceModel(), SIGNAL(rowsInserted(QModelIndex,int,int)), this, SLOT(sourceRowsInserted(QModelIndex,int,int)));
	connect(getSourceModel(), SIGNAL(columnsAboutToBeInserted(QModelIndex,int,int)), this, SLOT(sourceColumnsAboutToBeInserted(QModelIndex,int,int)));
	connect(getSourceModel(), SIGNAL(columnsInserted(QModelIndex,int,int)), this, SLOT(sourceColumnsInserted(QModelIndex,int,int)));
	connect(getSourceModel(), SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)), this, SLOT(sourceRowsAboutToBeRemoved(QModelIndex,int,int)));
	connect(getSourceModel(), SIGNAL(rowsRemoved(QModelIndex,int,int)), this, SLOT(sourceRowsRemoved(QModelIndex,int,int)));
	connect(getSourceModel(), SIGNAL(columnsAboutToBeRemoved(QModelIndex,int,int)), this, SLOT(sourceColumnsAboutToBeRemoved(QModelIndex,int,int)));
	connect(getSourceModel(), SIGNAL(columnsRemoved(QModelIndex,int,int)), this, SLOT(sourceColumnsRemoved(QModelIndex,int,int)));
	connect(getSourceModel(), SIGNAL(layoutAboutToBeChanged()), this, SLOT(sourceLayoutAboutToBeChanged()));
	connect(getSourceModel(), SIGNAL(layoutChanged()), this, SLOT(sourceLayoutChanged()));
	connect(getSourceModel(), SIGNAL(modelReset()), this, SLOT(sourceReset()));
	refresh();
}
*/

void QyurSqlTreeModel::refresh() {
	Q_D(QyurSqlTreeModel);
	emit beforeRefresh();
	d->sourceModel.submitAll();
	d->clear();
	reset();
	emit afterRefresh();
}

void QyurSqlTreeModel::sort(int column, Qt::SortOrder order) {
	Q_D(QyurSqlTreeModel);
	d->sourceModel.setSort(d->indexOfParid,(Qt::SortOrder)DROP_MULTIORDER);
	d->sourceModel.setSort(originalColumnByProxy(column),order);
	refresh();
}

bool QyurSqlTreeModel::setData(const QModelIndex& index, const QVariant& value, int role) {
	Q_D(QyurSqlTreeModel);
	if (d->sourceModel.setData(mapToSource(index),value,role)) {
		emit dataChanged(index,index);
		return true;
	}
	return false;
}

QyurSqlTreeModel::~QyurSqlTreeModel() {
	delete d_ptr;
}

const QSqlTableModel* QyurSqlTreeModel::getSourceModel() const {
	Q_D(const QyurSqlTreeModel);
	return &d->sourceModel;
}

int QyurSqlTreeModel::getSourceIdColumn() const {
	Q_D(const QyurSqlTreeModel);
	return d->indexOfId;
}

int QyurSqlTreeModel::getSourceParentIdColumn() const {
	Q_D(const QyurSqlTreeModel);
	return d->indexOfParid;
}

QModelIndex QyurSqlTreeModel::mapFromSource(const QModelIndex& sourceIndex) const {
	Q_D(const QyurSqlTreeModel);
	const int parid= sourceIndex.sibling(sourceIndex.row(), d->indexOfParid).data().toInt();
	const int id= sourceIndex.sibling(sourceIndex.row(), d->indexOfId).data().toInt();
	const int rowOfParid= d->getRowByParid(parid);
	int i= rowOfParid;
	for (; i<d->sourceModel.rowCount(); i++)
		if (d->sourceModel.record(i).field(d->indexOfId).value().toInt()==id)
			break;
	return createIndex(i-rowOfParid, proxyColumnByOriginal(sourceIndex.column()), d->getUserDataById(id));
}

QModelIndex QyurSqlTreeModel::mapToSource(const QModelIndex& proxyIndex) const {
	Q_D(const QyurSqlTreeModel);
	if (!d->proxy2source.contains(proxyIndex)) 
		d->proxy2source[proxyIndex]= d->sourceModel.index(d->getRowById(getIdByIndex(proxyIndex)),d->originalColumnByProxy(proxyIndex.column()));
	return d->proxy2source[proxyIndex];
}

bool QyurSqlTreeModel::hasChildren(const QModelIndex& index) const {
	Q_D(const QyurSqlTreeModel);
	if (!index.isValid())
		return true;
	return -1!=d->getRowByParid(getIdByIndex(index));
}

int QyurSqlTreeModel::columnCount(const QModelIndex&) const {
	Q_D(const QyurSqlTreeModel);
	return d->sourceModel.record().count();
}

QModelIndex QyurSqlTreeModel::index(int row, int column, const QModelIndex& parent) const {
	Q_D(const QyurSqlTreeModel);
	if (d->sourceModel.rowCount()==0)
		return QModelIndex();
	int id= parent.isValid()?getIdByIndex(parent):getRootParentId();
	if (!d->row2id.contains(QPair<int,int>(id,row))) {
		int i= 0;
		if (parent.isValid())
			i= d->getRowByParid(id);
		if (d->sourceModel.record(i+row).field(d->indexOfParid).value().toInt()==id)
			d->row2id[QPair<int,int>(id,row)]= d->sourceModel.record(i+row).field(d->indexOfId).value().toInt();
		else return QModelIndex();
	}
	return createIndex(row,column,d->getUserDataById(d->row2id[QPair<int,int>(id,row)]));//d->row2id[QPair(id,row)];
}

QModelIndex QyurSqlTreeModel::parent(const QModelIndex& index) const {
	Q_D(const QyurSqlTreeModel);
	if (d->sourceModel.rowCount()==0)
		return QModelIndex();
	if (!index.isValid())
		return QModelIndex();
	int parid= getParidByIndex(index);
	if (parid==getRootParentId())
		return QModelIndex();
	if (!d->id2ParentRow.contains(getIdByIndex(index))) {
		int paridOfParent= d->sourceModel.record(d->getRowById(parid)).field(d->indexOfParid).value().toInt();
		int rowOfParent=0;
		if (paridOfParent==getRootParentId()) {
			while (d->sourceModel.record(rowOfParent).field(d->indexOfParid).value().toInt()==getRootParentId()&&d->sourceModel.record(rowOfParent).field(d->indexOfId).value().toInt()!=parid)
				rowOfParent++;
		} else { 
			rowOfParent= d->getRowByParid(paridOfParent);
			if (rowOfParent>=0)
				while (d->sourceModel.record(rowOfParent).field(d->indexOfId).value().toInt()!=parid)
					rowOfParent++;
			rowOfParent-=d->getRowByParid(paridOfParent);
		}
		if (d->sourceModel.record(rowOfParent+((paridOfParent==getRootParentId())?0:d->getRowByParid(paridOfParent))).field(d->indexOfId).value().toInt()==parid)
			d->id2ParentRow[getIdByIndex(index)]= rowOfParent;
		else return QModelIndex();
	}
	return createIndex(d->id2ParentRow[getIdByIndex(index)],0,d->getUserDataById(parid));
}

int QyurSqlTreeModel::rowCount(const QModelIndex& parent) const {
	Q_D(const QyurSqlTreeModel);
	if (d->sourceModel.rowCount()==0)
		return 0;
	int id= parent.isValid()?getIdByIndex(parent):getRootParentId();
	if (!d->id2rowCount.contains(id)) {
		int i=0;
		if (!parent.isValid()) {
			while (d->sourceModel.record(i).field(d->indexOfParid).value().toInt()==getRootParentId()&&i<d->sourceModel.rowCount())
				i++;
		} else { 
			i= d->getRowByParid(getIdByIndex(parent));
			if (i>=0)
				while (d->sourceModel.record(i).field(d->indexOfParid).value().toInt()==getIdByIndex(parent))
					i++;
			i-=d->getRowByParid(getIdByIndex(parent));
		}
		d->id2rowCount[id]=i;
	}
	return d->id2rowCount[id];
}

bool QyurSqlTreeModel::removeRecords(const QModelIndexList& miList) {
	Q_D(QyurSqlTreeModel);
	// All childs must be removed with parent, so let walk through miList
	// and collect childs.
	QSet<QModelIndex> set= miList.toSet();
	QStack<QModelIndex> stack;
	foreach(QModelIndex index,set)
		stack.push(index);
	while(!stack.isEmpty()) {
		QModelIndex curr= stack.pop();
		set+=curr;
		for(int i=0; i<curr.model()->rowCount(curr); i++)
			stack.push(curr.child(i,0));
	}
	//now we have full list. 
	QModelIndexList list= set.toList();
	//sort list desc
	for(int i=1; i<list.size(); i++) {
		QModelIndex index= list[i];
		int j=i-1;
		while(j>=0&&mapToSource(list[j]).row()<mapToSource(index).row()) {
			list[j+1]=list[j];
			j--;
		}
		list[j+1]= index;
	}
	for (int i=0; i<list.size(); i++) 
		if (list[i].isValid())
			d->sourceModel.removeRows(mapToSource(list[i]).row(),1);
	d->sourceModel.submitAll();
	refresh();
	return true;
}

QModelIndex QyurSqlTreeModel::getIndexById(int id) const {
	Q_D(const QyurSqlTreeModel);
	QModelIndex parentIndex= (id>0)?getIndexById(d->getUserDataById(id)->parid):QModelIndex();
	for (int i=0; i<rowCount(parentIndex); i++) {
		if (getIdByIndex(index(i,0,parentIndex))==id)
			return index(i,0,parentIndex);
	}
	return QModelIndex();
}

int QyurSqlTreeModel::getIdByIndex(const QModelIndex& index) const {
	if (index.isValid())
		return static_cast<UserData*>(index.internalPointer())->id;
	return getRootParentId();
}

int QyurSqlTreeModel::getParidByIndex(const QModelIndex& index) const {
	return static_cast<UserData*>(index.internalPointer())->parid;
}

int QyurSqlTreeModel::getRootParentId() const {
	Q_D(const QyurSqlTreeModel);
	return d->rootParentId;
}

class QyurSqlTreeViewPrivate {
public:
};

QyurSqlTreeView::QyurSqlTreeView(): QTreeView(),d_ptr(new QyurSqlTreeViewPrivate){
	setSortingEnabled(true);
	setSelectionMode(QAbstractItemView::ExtendedSelection);
	setSelectionBehavior(QAbstractItemView::SelectRows);
}

QyurSqlTreeView::~QyurSqlTreeView() {
	delete d_ptr;
}

void QyurSqlTreeView::setModel(QAbstractItemModel* model) {
	QTreeView::setModel(model);
	connect(model, SIGNAL(afterRefresh()), SLOT(onAfterRefresh()));
	connect(this, SIGNAL(expanded(const QModelIndex&)), model, SLOT(addExpanded(const QModelIndex&)));
	connect(this, SIGNAL(collapsed(const QModelIndex&)), model, SLOT(addCollapsed(const QModelIndex&)));
	connect(selectionModel(), SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)), model, SLOT(onSelectionChanged(const QItemSelection&, const QItemSelection&)));
	connect(selectionModel(), SIGNAL(currentChanged(const QModelIndex&, const QModelIndex&)), model, SLOT(onCurrentChanged(const QModelIndex&)));
}

void QyurSqlTreeView::onAfterRefresh() {
	QyurSqlTreeModel* yurModel= qobject_cast<QyurSqlTreeModel*>(model());
	Q_CHECK_PTR(yurModel);
	disconnect(this, SIGNAL(expanded(const QModelIndex&)), yurModel, SLOT(addExpanded(const QModelIndex&)));
	disconnect(this, SIGNAL(collapsed(const QModelIndex&)), yurModel, SLOT(addCollapsed(const QModelIndex&)));
	disconnect(selectionModel(), SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)), yurModel, SLOT(onSelectionChanged(const QItemSelection&, const QItemSelection&)));
	disconnect(selectionModel(), SIGNAL(currentChanged(const QModelIndex&, const QModelIndex&)), yurModel, SLOT(onCurrentChanged(const QModelIndex&)));
	QModelIndexList expandedList= yurModel->expandedList(this);
	foreach(QModelIndex index, expandedList) {
		setExpanded(index,true);
	}
	QModelIndexList selectionList= yurModel->selectionList(selectionModel());
	foreach(QModelIndex index, selectionList)
		selectionModel()->select(index,QItemSelectionModel::Select|QItemSelectionModel::Rows);
	QModelIndex index= yurModel->current(selectionModel());
	selectionModel()->setCurrentIndex(index, QItemSelectionModel::SelectCurrent|QItemSelectionModel::Rows);
	connect(this, SIGNAL(expanded(const QModelIndex&)), yurModel, SLOT(addExpanded(const QModelIndex&)));
	connect(this, SIGNAL(collapsed(const QModelIndex&)), yurModel, SLOT(addCollapsed(const QModelIndex&)));
	connect(selectionModel(), SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)), yurModel, SLOT(onSelectionChanged(const QItemSelection&, const QItemSelection&)));
	connect(selectionModel(), SIGNAL(currentChanged(const QModelIndex&, const QModelIndex&)), yurModel, SLOT(onCurrentChanged(const QModelIndex&)));
	while ((index= index.parent()).isValid())
		emit expanded(index);
}

void QyurSqlTreeView::setColumnHidden(const QString& column, bool hide) {
	QTreeView::setColumnHidden(columnIndex(column),hide);
}

int QyurSqlTreeView::columnIndex(const QString& fieldName) const {
	return ((QyurSqlTreeModel*) model())->proxyColumnByOriginal(fieldName);
}

QModelIndex QyurSqlTreeView::addChild(const QVariantList& values, const QModelIndex& parent) {
	return qobject_cast<QyurSqlTreeModel*>(model())->addChild(values, parent, this);
}

void QyurSqlTreeView::remove(const QModelIndexList& list) {
	qobject_cast<QyurSqlTreeModel*>(model())->removeRecords(list);
}

#include "qyursqltreeview.moc"
