/*******************************************************
**
** Sample of using QyurSqlTreeView
**
** Copyright (C) 2009-2013 Yuri Yurachkovsky.
**
** QyurSqlTreeView is free software; you can redistribute
** it and/or modify it under the terms of the GNU
** Library General Public License as published by the
** Free Software Foundation; either version 2, or (at
** your option) any later version.
**
********************************************************/
#include "../qyursqltreeview.h"
#include <QtCore>
#include <QtGui>
#include <QtSql>

class SampleTreeView: public QyurSqlTreeView {
	Q_OBJECT
public:
	SampleTreeView();
	~SampleTreeView();
	void setModel(QyurSqlTreeModel* model);
protected slots:
	void onInsertRow(QSqlRecord&);
	void slotCustomContextMenuRequested(const QPoint&);
	void onModify();
	void onAdd();
	void onAddChild();
	void onRemove();
	void addChild(const QModelIndex& parent= QModelIndex());
private:
	QDialog* createDialog(const QModelIndex& index= QModelIndex());
	int idGenerator;
	QVector<QWidget*> editors;
	QMenu* menu;
	QAction* modifyAction, *addAction, *addChildAction, *removeAction;
};

SampleTreeView::SampleTreeView():QyurSqlTreeView(), idGenerator(100), menu(new QMenu) {
	setContextMenuPolicy(Qt::CustomContextMenu);
	connect(this,SIGNAL(customContextMenuRequested(const QPoint&)),SLOT(slotCustomContextMenuRequested(const QPoint&)));
	modifyAction= menu->addAction(tr("Modify"), this, SLOT(onModify()));
	addAction= menu->addAction(tr("AddSibling"), this, SLOT(onAdd()));
	addChildAction= menu->addAction(tr("AddChild"), this, SLOT(onAddChild()));
	removeAction= menu->addAction(tr("Remove"), this, SLOT(onRemove()));
}

SampleTreeView::~SampleTreeView() {
	
}

void SampleTreeView::setModel(QyurSqlTreeModel* model) {
	QyurSqlTreeView::setModel(model);
	connect(model->getSourceModel(), SIGNAL(beforeInsert(QSqlRecord&)),SLOT(onInsertRow(QSqlRecord&)));
}

void SampleTreeView::onInsertRow(QSqlRecord& record) {
	//Use this if id provided manually as in this sample 
	record.setValue("id", idGenerator++);
	//Use this if "id" provided by server
	//record.setGenerated("id", false);
}

void SampleTreeView::slotCustomContextMenuRequested(const QPoint&) {
	modifyAction->setEnabled(selectionModel()->selectedRows().size()==1);
	addAction->setEnabled(selectionModel()->selectedRows().size()<2);
	addChildAction->setEnabled(selectionModel()->selectedRows().size()==1);
	removeAction->setEnabled(selectionModel()->selectedRows().size()>0);
	menu->exec(QCursor::pos());
}

QDialog* SampleTreeView::createDialog(const QModelIndex& index) {
	QyurSqlTreeModel* yurModel= qobject_cast<QyurSqlTreeModel*>(model());
	QDialog* dialog= new QDialog;
	QVBoxLayout* topLayout= new QVBoxLayout(dialog);
	QGridLayout* dialogGridLayout= new QGridLayout;
	topLayout->addLayout(dialogGridLayout);
	QDialogButtonBox* buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
	QObject::connect(buttonBox, SIGNAL(accepted()), dialog, SLOT(accept()));
	QObject::connect(buttonBox, SIGNAL(rejected()), dialog, SLOT(reject())); 
	topLayout->addWidget(buttonBox);
	editors.clear();
	editors.resize(header()->count());
	for (int i=0; i<header()->count(); i++) {
		int dialogGridLayoutRow= dialogGridLayout->rowCount();
		QLabel* label= new QLabel(model()->headerData(i,Qt::Horizontal, Qt::DisplayRole ).toString());
		dialogGridLayout->addWidget(label ,dialogGridLayoutRow,0);
		QWidget* editor;
		int column= i;
		if (!index.isValid()) {
			column= yurModel->originalColumnByProxy(i);
			editor= QItemEditorFactory::defaultFactory()->createEditor(yurModel->getSourceModel()->record().field(column).type(),0);
		}
		else
			editor= itemDelegate()->createEditor(0,viewOptions(), index.sibling(index.row(), column));
		Q_CHECK_PTR(editor);
		editors[column]= editor;
		QMetaProperty property= editor->metaObject()->userProperty();
		property.write(editor,index.sibling(index.row(),column).data());
		dialogGridLayout->addWidget(editor,dialogGridLayoutRow,1);
		if (header()->isSectionHidden(i)) {
			label->hide(); editor->hide();
		}
		if (!i){
			if (qobject_cast<QLineEdit*>(editor))
				qobject_cast<QLineEdit*>(editor)->selectAll();
			editor->setFocus();
		}
	}
	return dialog;
}

void SampleTreeView::onModify() {
	QModelIndex current= selectionModel()->currentIndex();
	QDialog* dlg= createDialog(current);
	dlg->move(QCursor::pos());
	if (QDialog::Accepted==dlg->exec()) {
		for (int i=0; i<editors.size(); i++)
			itemDelegate()->setModelData(editors[i],model(),current.sibling(current.row(),i));
	}
	delete dlg;
}

void SampleTreeView::onAdd() {
	addChild(selectionModel()->currentIndex().parent());
}

void SampleTreeView::onAddChild() {
	addChild(selectionModel()->currentIndex());
}

void SampleTreeView::addChild(const QModelIndex& parent) {
	QyurSqlTreeModel* yurModel= qobject_cast<QyurSqlTreeModel*>(model());
	Q_CHECK_PTR(yurModel);
	QDialog* dlg= createDialog();
	dlg->move(QCursor::pos());
	if (QDialog::Accepted==dlg->exec()) {
		QVector<QVariant> vList;
		for (int i=0; i<editors.size(); i++) {
			int originalColumn= yurModel->originalColumnByProxy(i);
			if (vList.size()<=originalColumn)
				vList.resize(originalColumn+1);
			vList[originalColumn]= editors[originalColumn]->metaObject()->userProperty().read(editors[originalColumn]);
		}
		QyurSqlTreeView::addChild(vList.toList(), parent);
	}
	delete dlg;
}

void SampleTreeView::onRemove() {
	QModelIndexList selected= selectionModel()->selectedRows();
	if (QMessageBox::Yes==QMessageBox::question(0,QObject::tr("Warning!"),tr("Continue removing ")+QString::number(selected.size())+tr(" rows with childs?"),QMessageBox::Yes|QMessageBox::No)) {
		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
		remove(selected);
		QApplication::restoreOverrideCursor();
	}
}

int main(int argc, char *argv[]) {
	QApplication app(argc, argv);
	QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE");
	db.setDatabaseName(":memory:");
	if (!db.open()) {
		QMessageBox::critical(0, QObject::tr("Cannot open database. Program aborted."), QObject::tr("Database text: %1\nDriver text: %2").arg(db.lastError().databaseText(), db.lastError().driverText()), QMessageBox::Ok);
		return -1;
	}
	QSqlQuery query("create table treetable(id int default 0 primary key, parent_id int default -1, type varchar(20), title varchar(20), summary varchar(20))");
	QyurSqlTreeModel model("treetable",QStringList()<<QObject::tr("TITLE")<<QObject::tr("SUMMARY")<<QObject::tr("TYPE")<<QObject::tr("ID")<<QObject::tr("PARENTID"),QStringList()<<"title"<<"summary"<<"type"<<"id"<<"parent_id","id","parent_id");
	QWidget topWidget;
	topWidget.setWindowTitle("QyurSqlTreeView Demo");
	QHBoxLayout topLayout(&topWidget);
	QSplitter mainSplitter;
	QTableView tableView;
	tableView.setSelectionBehavior(QAbstractItemView::SelectRows);
	tableView.setSelectionMode(QAbstractItemView::SingleSelection);
	tableView.setModel(model.sourceModel());
	mainSplitter.addWidget(&tableView);
	QHBoxLayout vlayout(&mainSplitter);
	SampleTreeView treeView;
	vlayout.addWidget(&treeView);
	treeView.setModel(&model);
	treeView.sortByColumn(treeView.columnIndex("id"),Qt::AscendingOrder);
	treeView.setColumnHidden("id", true); 
	treeView.setColumnHidden("parent_id", true);
	treeView.setColumnHidden("type", true); 
	topLayout.addWidget(&mainSplitter);
	topWidget.show();
	return app.exec();
}

#include "main.moc"
